<?php

class PieceSquadro {
    const BLANC = 0;
    const NOIR = 1;
    const VIDE = 2;
    const NEUTRE = 3;
    const NORD = 4;
    const EST = 5;
    const SUD = 6;
    const OUEST = 7;

    private $couleur;
    private $direction;

    /*Constructeur privé : Le constructeur est privé,
    ce qui est une bonne pratique pour forcer l'utilisation
    des méthodes statiques (initVide, initNoirNord, etc.). Cependant, il n'y a pas de vérification des valeurs passées au constructeur. Par exemple, si une couleur ou une direction invalide est passée, cela pourrait causer des problèmes.*/

    //Solution : Ajouter une validation dans le constructeur pour s'assurer que les valeurs de couleur et de direction sont valides.

    private function __construct($couleur, $direction) {
        $this->couleur = $couleur;
        $this->direction = $direction;
    }

    public static function initVide() {
        return new self(self::VIDE, self::NORD);
    }

    public static function initNoirNord() {
        return new self(self::NOIR, self::NORD);
    }

    public static function initNoirSud() {
        return new self(self::NOIR, self::SUD);
    }

    public static function initBlancEst() {
        return new self(self::BLANC, self::EST);
    }

    public static function initBlancOuest() {
        return new self(self::BLANC, self::OUEST);
    }

    public function getCouleur() {
        return $this->couleur;
    }

    public function getDirection() {
        return $this->direction;
    }

    //La méthode inverse la direction entre NORD et SUD, mais elle ne gère pas les autres directions (EST et OUEST). Cela pourrait poser problème si une pièce a une direction autre que NORD ou SUD.
    //Solution : Vérifier si la direction est NORD ou SUD avant de l'inverser, ou gérer toutes les directions possibles.
    public function inverseDirection() {
        $this->direction = ($this->direction === self::NORD) ? self::SUD : self::NORD;
    }

    public function toJson() {
        return json_encode(['couleur' => $this->couleur, 'direction' => $this->direction]);
    }

    public static function fromJson($json) {
        $data = json_decode($json, true);
        return new self($data['couleur'], $data['direction']);
    }
}
?>
